const mongoose = require('mongoose');
const express = require('express');
const cookieParser = require('cookie-parser');
const fs = require('fs');
const crypto = require('crypto');

const connection_string = 'mongodb://127.0.0.1/responsibilities';

mongoose.connect(connection_string);
mongoose.connection.on('error', () => {
  console.log('There was a problem connecting to mongoDB');
});

/**
 * Specify the schema for the database, including items and categories.
 */
 var UserSchema = new mongoose.Schema({
  username: String,
  hash: String,
  salt: String,
  faveColor: String
});
var User = mongoose.model('User', UserSchema );
var ItemSchema = new mongoose.Schema( {
    name: String,
    difficulty: Number,
    stat: String,
    username: String,
    category: String
});
var Item = mongoose.model('Item', ItemSchema);
var CategorySchema = new mongoose.Schema( {
  name: String,
  priority: Number,
  color: String,
  username: String
});
var Category = mongoose.model('Category', CategorySchema);

let sessions = {};

function addSession(username) {
  let sid = Math.floor(Math.random() * 1000000000);
  let now = Date.now();
  sessions[username] = {id: sid, time: now};
  return sid;
}

function removeSessions() {
  let now = Date.now();
  let usernames = Object.keys(sessions);
  for (let i = 0; i < usernames.length; i++) {
    let last = sessions[usernames[i]].time;
    //if (last + 120000 < now) {
    if (last + 20000 < now) {
      delete sessions[usernames[i]];
    }
  }
  console.log(sessions);
}

setInterval(removeSessions, 2000);

/**
 * Initialize the express app and configure with various features 
 * such as JSON parsing, static file serving, etc.
 */
const app = express();

// Add headers before the routes are defined
app.use(function (req, res, next) {

    res.setHeader('Access-Control-Allow-Origin', 'http://localhost:3000');
    res.setHeader('Access-Control-Allow-Credentials', 'true');

    console.log('CORS');

    // Pass to next layer of middleware
    next();
});

app.use(cookieParser());    
//app.get('/', (req, res) => { res.redirect('/app/index.html'); });
app.use(express.json())
//app.use(parser.text({type: '*/*'}));

function isloggedinnow(req, res, next) {
  let c = req.cookies;
  console.log('auth request:');
  console.log(req.cookies);
  if (c != undefined && c.login != undefined) {
    if (sessions[c.login.username] != undefined && 
      sessions[c.login.username].id == c.login.sessionID) {
        return true;
    } else {
      return false;
    }
  } else {
    return false;
  }
  return false;
}

function authenticate(req, res, next) {
  let c = req.cookies;
  console.log('auth request:');
  console.log(req.cookies);
  if (c != undefined) {
    if (sessions[c.login.username] != undefined && 
      sessions[c.login.username].id == c.login.sessionID) {
      next();
    } else {
      res.redirect('/account/index.html');
    }
  }  else {
    res.redirect('/account/index.html');
  }
}

app.use('/app/*', authenticate);

app.use(express.static('public_html'))

app.get('/isloggedin', (req, res, next) => {
  console.log(req.cookies);
  if (isloggedinnow(req, res, next)) {
    console.log('yes');
    res.end('yes');
  } else {
    console.log('no');
    res.end('no');
  }
});

/**
 * Get all of the items that correspond with a particular catagory.
 * The category is specified as a part of the URL after /items/.
 */
app.get('/items', (req, res, next) => {
    let cat = req.params.cat;
    let p1 = Item.find().exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      res.end('FAIL');
    });
});

/**
 * This route is for creating a new item for a particular category. 
 * The category that the items belongs to is stored as a part of the item object in the database.
 */
app.get('/create/item/:n/:s/:c', (req, res) => {
  let newItemToSave = req.body;
  var newItem = new Item( {name: req.params.n,
    stat: req.params.s, 
    category: req.params.c});
  let p1 = newItem.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAIL');
  });
});

/**
 * This route is for creating a new list category.
 */
app.post('/create/category/', (req, res) => {
  let CategoryToSave = req.body;
  var newCategory = new Category(CategoryToSave);
  let p1 = newCategory.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAILED TO CREATE A CATEGORY');
  });
});

/**
 * This route is for fetching all of the categories stored in the database.
 */
app.get('/categories/', (req, res) => {
    let p1 = Category.find({}).exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      console.log(error);
      res.end('FAIL');
    });
});

app.get('/account/login/:user/:pass', (req, res) => { 
  console.log(sessions);
  let u = req.params;
  let p1 = User.find({username: u.user}).exec();
  p1.then( (results) => { 
    if (results.length == 0) {
      res.end('Coult not find account');
    } else {
      let currentUser = results[0];
      let toHash = u.pass + currentUser.salt;
      let h = crypto.createHash('sha3-256');
      let data = h.update(toHash, 'utf-8');
      let result = data.digest('hex');
      
      console.log(toHash);
      console.log('HASH WE JUST MADE:');
      console.log(result);
      console.log('THE ORIGINAL HASH:');
      console.log(currentUser.hash);

      if (result == currentUser.hash) {
          let sid = addSession(u.username);  
          res.cookie("login", 
            {username: u.username, sessionID: sid}, 
            {maxAge: 60000 * 2 });
          res.end('SUCCESS');
      } else {
          res.end('FAILED TO LOG IN');
      }
    }
  });
});

app.get('/account/create/:user/:pass', (req, res) => { 
  
  let p1 = User.find({username: req.params.user}).exec();
  p1.then( (results) => { 
    if (results.length == 0) {
      let newSalt = '' + Math.floor(Math.random() * 10000000000);
      let toHash = req.params.pass + newSalt;
      let h = crypto.createHash('sha3-256');
      let data = h.update(toHash, 'utf-8');
      let result = data.digest('hex');
     
       /*
      console.log(req.params.pass);
      console.log(newSalt);
      console.log(toHash);
      console.log(result);
      */
      let u = new User({ 
        username: req.params.user,
        hash: result,
        salt: newSalt,
        faveColor: 'red'
      });
      let p = u.save();
      p.then(() => { 
        res.end('USER CREATED');
      });
      p.catch( () => { 
        res.end('DATABASE SAVE ISSUE');
      });
    } else {
      res.end('USERNAME ALREADY TAKEN');
    }
  });

});
// Start up the server to listen on port 80
const port = 80;
app.listen(port, () => { console.log('server has started'); });

